@@ -1,20 +1,17 @@
-import autoarray as aa
-import autofit as af
-from autoarray.plot import mat_objs
-from autoastro.plot import lensing_plotters
-from autoastro.plot import fit_galaxy_plots
-from autolens.plot import (
-    ray_tracing_plots,
-    hyper_plots,
-    fit_imaging_plots,
-    fit_interferometer_plots,
-    inversion_plots,
-)
 import copy
+from scipy.stats import norm
+import matplotlib.pyplot as plt
+
+from autoconf import conf
+import autoarray as aa
+from autoarray.plot import mat_objs
+from autogalaxy.plot import fit_galaxy_plots, hyper_plots, inversion_plots
+from autogalaxy.plot import lensing_plotters
+from autolens.plot import ray_tracing_plots, fit_imaging_plots, fit_interferometer_plots
 
 
 def setting(section, name):
-    return af.conf.instance.visualize_plots.get(section, name, bool)
+    return conf.instance.visualize_plots.get(section, name, bool)
 
 
 def plot_setting(section, name):
@@ -41,9 +38,7 @@ class AbstractVisualizer:
         self.plot_subplot_ray_tracing = plot_setting(
             "ray_tracing", "subplot_ray_tracing"
         )
-        self.plot_ray_tracing_profile_image = plot_setting(
-            "ray_tracing", "profile_image"
-        )
+        self.plot_ray_tracing_image = plot_setting("ray_tracing", "image")
         self.plot_ray_tracing_source_plane = plot_setting(
             "ray_tracing", "source_plane_image"
         )
@@ -181,6 +176,8 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
         self.plot_hyper_model_image = plot_setting("hyper_galaxy", "model_image")
         self.plot_hyper_galaxy_images = plot_setting("hyper_galaxy", "images")
 
+        self.plot_stochastic_histogram = plot_setting("other", "stochastic_histogram")
+
     def visualize_ray_tracing(self, tracer, during_analysis):
 
         plotter = self.plotter.plotter_with_new_output(
@@ -205,7 +202,7 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
             positions=self.include.positions_from_masked_dataset(
                 masked_dataset=self.masked_dataset
             ),
-            plot_profile_image=self.plot_ray_tracing_profile_image,
+            plot_image=self.plot_ray_tracing_image,
             plot_source_plane=self.plot_ray_tracing_source_plane,
             plot_convergence=self.plot_ray_tracing_convergence,
             plot_potential=self.plot_ray_tracing_potential,
@@ -225,7 +222,7 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
                     positions=self.include.positions_from_masked_dataset(
                         masked_dataset=self.masked_dataset
                     ),
-                    plot_profile_image=True,
+                    plot_image=True,
                     plot_source_plane=True,
                     plot_convergence=True,
                     plot_potential=True,
@@ -247,7 +244,7 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
         ray_tracing_plots.individual(
             tracer=tracer,
             grid=self.masked_dataset.grid,
-            plot_profile_image=True,
+            plot_image=True,
             plot_source_plane=True,
             plot_convergence=True,
             plot_potential=True,
@@ -286,6 +283,34 @@ class PhaseDatasetVisualizer(AbstractVisualizer):
                 sub_plotter=sub_plotter,
             )
 
+    def visualize_stochastic_histogram(
+        self, log_evidences, max_log_evidence, during_analysis
+    ):
+
+        if log_evidences is None:
+            return
+
+        plotter = self.plotter.plotter_with_new_output(
+            path=self.plotter.output.path + "other/"
+        )
+
+        if self.plot_stochastic_histogram and not during_analysis:
+
+            bins = conf.instance.general.get(
+                "inversion", "stochastic_histogram_bins", int
+            )
+
+            (mu, sigma) = norm.fit(log_evidences)
+            n, bins, patches = plt.hist(x=log_evidences, bins=bins, density=1)
+            y = norm.pdf(bins, mu, sigma)
+            plt.plot(bins, y, "--")
+            plt.xlabel("log evidence")
+            plt.title("Stochastic Log Evidence Histogram")
+            plt.axvline(max_log_evidence, color="r")
+            plt.savefig(
+                f"{plotter.output.path}stochastic_histogram.png", bbox_inches="tight"
+            )
+
 
 class PhaseImagingVisualizer(PhaseDatasetVisualizer):
     def __init__(self, masked_dataset, image_path, results=None):
@@ -385,10 +410,10 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
                     grid=self.include.inversion_image_pixelization_grid_from_fit(
                         fit=fit
                     ),
-                    light_profile_centres=self.include.light_profile_centres_of_galaxies_from_obj(
+                    light_profile_centres=self.include.light_profile_centres_from_obj(
                         obj=fit.tracer.image_plane
                     ),
-                    mass_profile_centres=self.include.mass_profile_centres_of_galaxies_from_obj(
+                    mass_profile_centres=self.include.mass_profile_centres_from_obj(
                         obj=fit.tracer.image_plane
                     ),
                     critical_curves=self.include.critical_curves_from_obj(
@@ -410,10 +435,10 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
                     fit=fit, plane_index=-1
                 ),
                 grid=self.include.inversion_image_pixelization_grid_from_fit(fit=fit),
-                light_profile_centres=self.include.light_profile_centres_of_galaxies_from_obj(
+                light_profile_centres=self.include.light_profile_centres_from_obj(
                     obj=fit.tracer.image_plane
                 ),
-                mass_profile_centres=self.include.mass_profile_centres_of_galaxies_from_obj(
+                mass_profile_centres=self.include.mass_profile_centres_from_obj(
                     obj=fit.tracer.image_plane
                 ),
                 critical_curves=self.include.critical_curves_from_obj(obj=fit.tracer),
@@ -460,10 +485,10 @@ class PhaseImagingVisualizer(PhaseDatasetVisualizer):
                         grid=self.include.inversion_image_pixelization_grid_from_fit(
                             fit=fit
                         ),
-                        light_profile_centres=self.include.light_profile_centres_of_galaxies_from_obj(
+                        light_profile_centres=self.include.light_profile_centres_from_obj(
                             obj=fit.tracer.image_plane
                         ),
-                        mass_profile_centres=self.include.mass_profile_centres_of_galaxies_from_obj(
+                        mass_profile_centres=self.include.mass_profile_centres_from_obj(
                             obj=fit.tracer.image_plane
                         ),
                         critical_curves=self.include.critical_curves_from_obj(
@@ -608,10 +633,10 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
             #             fit=fit, plane_index=-1
             #         ),
             #         grid=self.include.inversion_image_pixelization_grid_from_fit(fit=fit),
-            #         light_profile_centres=self.include.light_profile_centres_of_galaxies_from_obj(
+            #         light_profile_centres=self.include.light_profile_centres_of_planes_from_obj(
             #             obj=fit.tracer.image_plane
             #         ),
-            #         mass_profile_centres=self.include.mass_profile_centres_of_galaxies_from_obj(
+            #         mass_profile_centres=self.include.mass_profile_centres_of_planes_from_obj(
             #             obj=fit.tracer.image_plane
             #         ),
             #         critical_curves=self.include.critical_curves_from_obj(obj=fit.tracer),
@@ -627,10 +652,10 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
                     fit=fit, plane_index=-1
                 ),
                 grid=self.include.inversion_image_pixelization_grid_from_fit(fit=fit),
-                light_profile_centres=self.include.light_profile_centres_of_galaxies_from_obj(
+                light_profile_centres=self.include.light_profile_centres_from_obj(
                     obj=fit.tracer.image_plane
                 ),
-                mass_profile_centres=self.include.mass_profile_centres_of_galaxies_from_obj(
+                mass_profile_centres=self.include.mass_profile_centres_from_obj(
                     obj=fit.tracer.image_plane
                 ),
                 critical_curves=self.include.critical_curves_from_obj(obj=fit.tracer),
@@ -674,10 +699,10 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
                         grid=self.include.inversion_image_pixelization_grid_from_fit(
                             fit=fit
                         ),
-                        light_profile_centres=self.include.light_profile_centres_of_galaxies_from_obj(
+                        light_profile_centres=self.include.light_profile_centres_from_obj(
                             obj=fit.tracer.image_plane
                         ),
-                        mass_profile_centres=self.include.mass_profile_centres_of_galaxies_from_obj(
+                        mass_profile_centres=self.include.mass_profile_centres_from_obj(
                             obj=fit.tracer.image_plane
                         ),
                         critical_curves=self.include.critical_curves_from_obj(
@@ -725,10 +750,10 @@ class PhaseInterferometerVisualizer(PhaseDatasetVisualizer):
                         grid=self.include.inversion_image_pixelization_grid_from_fit(
                             fit=fit
                         ),
-                        light_profile_centres=self.include.light_profile_centres_of_galaxies_from_obj(
+                        light_profile_centres=self.include.light_profile_centres_from_obj(
                             obj=fit.tracer.image_plane
                         ),
-                        mass_profile_centres=self.include.mass_profile_centres_of_galaxies_from_obj(
+                        mass_profile_centres=self.include.mass_profile_centres_from_obj(
                             obj=fit.tracer.image_plane
                         ),
                         critical_curves=self.include.critical_curves_from_obj(
