@@ -1,8 +1,8 @@
-import autolens as al
-
-import numpy as np
 import time
 
+import autolens as al
+import numpy as np
+
 repeats = 1
 
 total_visibilities = 100000
@@ -41,7 +41,7 @@ print("Real space circular mask radius = " + str(real_space_radius) + "\n")
 lens_galaxy = al.Galaxy(
     redshift=0.5,
     mass=al.mp.EllipticalIsothermal(
-        centre=(0.0, 0.0), einstein_radius=1.6, axis_ratio=0.7, phi=45.0
+        centre=(0.0, 0.0), einstein_radius=1.6, elliptical_comps=(0.17647, 0.0)
     ),
 )
 
